
/*************************************************************
*                                                            *
*       CGI C++ Library v0.30a                               *
*                                                            *
*       (c) 2005-2007 by Ighor Poteryakhin (ighor@ighor.ru)  *
*                                                            *
*       cgi.cpp                                              *
*                                                            *
*************************************************************/

#include "cgi.h"

using namespace std;

namespace cgi{

// class cgi

    cgi_app::cgi_app()
    {
	init();
    }

    cgi_app::cgi_app(codepage cp)
    {
	init();
	setcp(cp);
    }

    void cgi_app::init()
    {
	if(getenv("REQUEST_METHOD")==NULL)
	    throw "Non-CGI call?!";

	if(server("HTTP_COOKIE")!=NULL)
	    _cookies_string=server("HTTP_COOKIE");
	_split_cookies(_cookies_string,_cookie);

	if(!strcmp(getenv("REQUEST_METHOD"),"GET"))
	{
	    string str;
	    while(cin>>str)
		_input.push_back(str);

	    if(server("QUERY_STRING")!=NULL)
		_query_string=server("QUERY_STRING");
	    _setmethod(M_GET);
	    _split_parametrs(_query_string,_get);
	}
	else if(!strcmp(getenv("REQUEST_METHOD"),"POST"))
	{
    	    _setmethod(M_POST);
	    if(!strcmp(server("CONTENT_TYPE"),"application/x-www-form-urlencoded"))
	    {
		string str;
		while(cin>>str)
		    _input.push_back(str);

		if(atoi(server("CONTENT_LENGTH"))!=_input[0].length())
		    throw "Internal error: cgi_app::init() CONTENT_LENGTH does not equal to read data !!!";
		_split_parametrs(_input[0],_post);
	    }
	    else
	    {
		string str;
		string boundary;

		cin>>boundary;

		string filename, name, value;
		while(cin>>str)
		{
		    if(str=="Content-Disposition")
			{ cin>>str; continue; }
		    else if(str==boundary)
			continue;
		    else if(str.substr(0,5)=="name=")
		    {
			name=str.substr(5,str.length()-5);
			if(name.length())
			    while(name.substr(0,1)=="\"")
				name=name.substr(1,name.length()-1);
			if(name.length())
			    while(name.substr(name.length()-1,1)=="\""||name.substr(name.length()-1,1)==";")
				name=name.substr(0,name.length()-1);
			cin>>value;
		   	if(value.substr(0,9)=="filename=")
			{
				filename=value.substr(9,value.length()-9);
				if(filename.length())
				    while(filename.substr(0,1)=="\"")
					filename=filename.substr(1,filename.length()-1);
				if(filename.length())
				    while(filename.substr(filename.length()-1,1)=="\""||filename.substr(filename.length()-1,1)==";")
					filename=filename.substr(0,filename.length()-1);

				_post.insert(parametrs::value_type( name, filename));
				_post.insert(parametrs::value_type( "filename", filename));

				cin>>str; cin>>str; // Ignore content type

				filename=consts::TMPDIR+filename;
            			std::ofstream outfile(filename.c_str(),ios_base::out|ios_base::trunc|ios_base::binary);
            			std::string buf;
            			
            			std::string div=string("\r\n")+boundary; // skips \r\n before closing boundary
            			char ch;
            			int c=0;

            			do
            			{
            			    cin.read((char*)&ch,sizeof(ch));
            			    if(c++<4) // skip first \r\n
            				if(ch=='\r'||ch=='\n')
            				    continue;
            			    buf+=ch;
            			    if(buf.length()>div.length())
            			    {
            				ch=buf[0];
            				outfile.write((char*)&ch,sizeof(ch));
            				buf=buf.substr(1,buf.length()-1);
            			    }
            			}
            			while(buf!=div); //  true
			}
			else
			    _post.insert(parametrs::value_type( name, value ));
		    }
		}
	    }
	}
	else 
	{
	    throw "Error: Only GET and POST methods are supported...";
	}
	if(server("HTTPS")!=NULL)
	{
	    std::string https(server("HTTPS"));
	    if(https=="on")
		_ssl=true;
	    else
		_ssl=false;
	}
	else
	    _ssl=false;

	if(cookie("SESSION_ID")!=NULL)
	    _session_id=cookie("SESSION_ID");
	else
	    _session_id="";

	_basic_auth_user="";
	_basic_auth_password="";
	std::string auth;
	if(server("REMOTE")!=NULL)
	    auth=server("REMOTE");
	if(auth.length()>1)
	{
	    auth=server("REMOTE");
	    auth=auth.substr(auth.find("Basic ")+string("Basic ").length(),-1);
	    auth=base64_decode(auth);
	    _basic_auth_user=auth.substr(0,auth.find(":"));
	    _basic_auth_password=auth.substr(auth.find(":")+1,-1);
	}
    }

    void cgi_app::_split_parametrs(string& s, parametrs& p)
    {
	// r=1&t=2 -> map( ("r","1"),("t","2") )
	string in(s);
	string inpart;
	string name;
	string value;
	
	while(in.length())
	{
	    if(in.find("&")!=string::npos)
	    {
		inpart=in.substr(0,in.find("&"));
		in.erase(0,in.find("&")+1);
	    }
	    else
	    {
		inpart=in;
		in="";
	    }

	    if(inpart.find("=")!=string::npos)
	    {
		name=inpart.substr(0,inpart.find("="));
		value=inpart.substr(inpart.find("=")+1,inpart.length()-inpart.find("=")-1);
	    }
	    else
	    {
		name=inpart;
		value="";
	    }
	    value=decoder.decode(value);
	    p.insert(parametrs::value_type( name, value ));
	}
/*
	ofstream debug;
	debug.open("DEBUG",ofstream::out | ofstream::app);
	for(parametrs::iterator i=p.begin();i!=p.end();i++)
	{
	    debug<<i->first<<" "<<i->second<<endl;
	}
*/
    }


    void cgi_app::_split_cookies(string& s, parametrs& p)
    {
	// r=1; t=2 -> map( ("r","1"),("t","2") )
	string in(s);
	string inpart;
	string name;
	string value;
	
	while(in.length())
	{
	    if(in.find("; ")!=string::npos)
	    {
		inpart=in.substr(0,in.find("; "));
		in.erase(0,in.find("; ")+2);
	    }
	    else
	    {
		inpart=in;
		in="";
	    }

	    if(inpart.find("=")!=string::npos)
	    {
		name=inpart.substr(0,inpart.find("="));
		value=inpart.substr(inpart.find("=")+1,inpart.length()-inpart.find("=")-1);
	    }
	    else
	    {
		name=inpart;
		value="";
	    }
	    value=decoder.decode(value);
	    p.insert(parametrs::value_type( name, value ));
	}
    }

    cgi_app::~cgi_app()
    {
	if(!_printed)
	    print();
    }

    methods cgi_app::method()
    {
	return _method;
    }

    char* cgi_app::smethod()
    {
	switch(method())
	{
	    case M_GET:
		return "GET";
	    case M_POST:
		return "POST";
	}
    }

    void cgi_app::_setmethod(methods method)
    {
	_method=method;
    }

    char* cgi_app::get(const char* name)
    {
	if(method()==M_POST)
	    throw "Error while get(): Current method is POST!";
	if(_get.find(name)==_get.end())
	    return NULL;
	else
	    return (char*)_get[name].c_str();
    }

    char* cgi_app::post(const char* name)
    {
	if(method()==M_GET)
	    throw "Error while post(): Current method is GET!";
	if(_post.find(name)==_post.end())
	    return NULL;
	else
	    return (char*)_post[name].c_str();
    }

    char* cgi_app::cookie(const char* name)
    {
	if(_cookie.find(name)==_cookie.end())
	    return NULL;
	else
	    return (char*)_cookie[name].c_str();
    }

    char* cgi_app::server(const char* name)
    {
	return getenv(name);
    }

    
    void cgi_app::setcookie(cookie_type& c)
    {
	_cookies_set.push_back(c);
	//Set-Cookie: name=value; expires=date; path=path; domain=domain_name
	// ; secure - for SSL connections only
	string s;
	s="Set-Cookie: ";
	s+=c.name;
	s+="=";
	s+=encoder.encode(c.value);
	s+="; expires=";
	s+=_gettime(c.expires);
	s+="; path=";
	s+=c.path;
	s+="; domain=";
	s+=c.domain;
	if(ssl())
	    s+="; secure";
	header(s);
    }

    char* cgi_app::start_session(void)
    {
	if(_printed)
	    throw "Already printed!";
	//...
	std::string id(session_id());
	cookie_type cookie;
	if(id!="")
	    cookie.value=session_id();
	else
	    cookie.value=server("UNIQUE_ID");
	cookie.name="SESSION_ID";
	time_t t;
	time(&t);
	cookie.expires=t+consts::session_duration;
	cookie.path="/";
	cookie.domain=server("HTTP_HOST");
	setcookie(cookie);

	_session_id=cookie.value;
	return session_id();
    }
    char* cgi_app::session_id()
    {
	return (char*)_session_id.c_str();
    }
    inline bool cgi_app::ssl()
    {
	return _ssl;
    }

    const char* cgi_app::basic_auth_user()
    {
	return _basic_auth_user.c_str();
    }
    const char* cgi_app::basic_auth_password()
    {
	return _basic_auth_password.c_str();
    }
}
