#ifndef CGI_H
#define CGI_H

/*************************************************************
*                                                            *
*       CGI C++ Library v0.29a                               *
*                                                            *
*       (c) 2005-2007 by Ighor Poteryakhin (ighor@ighor.ru)  *
*                                                            *
*       cgi.h                                                *
*                                                            *
*************************************************************/

#include "document.h"
#include "urlcode.h"
#include "const.h"

#include <cstdlib>

#include <map>
#include <string>
/*#include <sstream>*/

namespace cgi
{
    enum methods	{M_GET, M_POST};
    
    typedef std::map<std::string, std::string > parametrs;
    typedef std::deque<std::string> text;
    typedef struct {std::string name;
		    std::string value; 
		    time_t expires;
		    std::string path;
		    std::string domain;} cookie_type;
    typedef std::deque<cookie_type> cookies;

    // the main class - cgi_app - use it to create cgi applications
    class cgi_app: public document
    {
	public:
    	    cgi_app();
    	    cgi_app(codepage);
	    ~cgi_app();

	    void init();

	    methods method();
	    char* smethod();

	    inline bool	ssl(); 		// true if connection over SSL/TLS

	    char* get(const char*);
	    char* post(const char*);
	    char* cookie(const char*);
	    char* server(const char*);

	    void setcookie(cookie_type&);
	    
	    urlcode::urlencode encoder;
	    urlcode::urldecode decoder;
	    
	    char* start_session(void);
	    char* session_id(void);
	    
// To use this you need to set in your .htaccess file
// RewriteEngine On
// RewriteRule .* - [E=REMOTE:%{HTTP:Authorization},L]
// This works only with AUTH_TYPE=="Basic"

	    const char* basic_auth_user();
	    const char* basic_auth_password();
// -----

	private:
	    std::string _session_id;

	    parametrs	_get;		// GET variables in map <string, string>
	    parametrs	_post;		// POST variables in map <string, string>
	    parametrs	_cookie;
	    text	_input;

	    void _setmethod(const methods method);
	    void _split_parametrs (std::string&,parametrs&);
	    void _split_cookies (std::string&,parametrs&);

	    std::string	_query_string;
	    std::string	_cookies_string; //string with cookies, that was sent to us
	    methods	_method;
	    cookies	_cookies_set;	// "Out" cookies (set by setcookie())
	    bool	_ssl; 		// true if connection over SSL/TLS

	    std::string _basic_auth_user;
	    std::string _basic_auth_password;
	    
    };

}

#endif
